#include "object.h"
#include "objectinstance.h"
#include "instancescene.h"
#include "renderer.h"

namespace Msp {
namespace GL {

void InstanceScene::add(const Renderable &r)
{
	renderables[r.get_instance_key()].insert(&r);
}

void InstanceScene::remove(const Renderable &r)
{
	InstanceMap::iterator i = renderables.find(r.get_instance_key());
	if(i!=renderables.end())
	{
		i->second.erase(&r);
		if(i->second.empty())
			renderables.erase(i);
	}
}

void InstanceScene::setup_frame() const
{
	for(InstanceMap::const_iterator i=renderables.begin(); i!=renderables.end(); ++i)
		for(RenderableSet::const_iterator j=i->second.begin(); j!=i->second.end(); ++j)
			(*j)->setup_frame();
}

void InstanceScene::finish_frame() const
{
	for(InstanceMap::const_iterator i=renderables.begin(); i!=renderables.end(); ++i)
		for(RenderableSet::const_iterator j=i->second.begin(); j!=i->second.end(); ++j)
			(*j)->finish_frame();
}

void InstanceScene::render(Renderer &renderer, const Tag &tag) const
{
	if(setup_frustum(renderer))
	{
		for(InstanceMap::const_iterator i=renderables.begin(); i!=renderables.end(); ++i)
			for(RenderableSet::const_iterator j=i->second.begin(); j!=i->second.end(); ++j)
				if(!frustum_cull(**j))
					renderer.render(**j, tag);
	}
	else
	{
		for(InstanceMap::const_iterator i=renderables.begin(); i!=renderables.end(); ++i)
			for(RenderableSet::const_iterator j=i->second.begin(); j!=i->second.end(); ++j)
				renderer.render(**j, tag);
	}
}

} // namespace GL
} // namespace Msp
