#include "programsyntax.h"

using namespace std;

namespace Msp {
namespace GL {
namespace ProgramSyntax {

Block::Block():
	use_braces(false)
{ }

void Block::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


void Literal::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


void ParenthesizedExpression::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


VariableReference::VariableReference():
	declaration(0)
{ }

void VariableReference::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


void MemberAccess::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


UnaryExpression::UnaryExpression():
	prefix(true)
{ }

void UnaryExpression::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


BinaryExpression::BinaryExpression():
	assignment(false)
{ }

void BinaryExpression::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


FunctionCall::FunctionCall():
	constructor(false)
{ }

void FunctionCall::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


void ExpressionStatement::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


void Layout::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


StructDeclaration::StructDeclaration()
{
	members.use_braces = true;
}

void StructDeclaration::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


VariableDeclaration::VariableDeclaration():
	constant(false),
	type_declaration(0),
	array(false),
	linked_declaration(0)
{ }

void VariableDeclaration::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


InterfaceBlock::InterfaceBlock()
{
	members.use_braces = true;
}

void InterfaceBlock::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


FunctionDeclaration::FunctionDeclaration():
	definition(false)
{ }

void FunctionDeclaration::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


void Conditional::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


void Iteration::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


void Passthrough::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


void Return::visit(NodeVisitor &visitor)
{
	visitor.visit(*this);
}


void TraversingVisitor::visit(Block &block)
{
	for(list<NodePtr<Node> >::iterator i=block.body.begin(); i!=block.body.end(); ++i)
		(*i)->visit(*this);
}

void TraversingVisitor::visit(ParenthesizedExpression &parexpr)
{
	parexpr.expression->visit(*this);
}

void TraversingVisitor::visit(MemberAccess &memacc)
{
	memacc.left->visit(*this);
}

void TraversingVisitor::visit(UnaryExpression &unary)
{
	unary.expression->visit(*this);
}

void TraversingVisitor::visit(BinaryExpression &binary)
{
	binary.left->visit(*this);
	binary.right->visit(*this);
}

void TraversingVisitor::visit(FunctionCall &call)
{
	for(vector<NodePtr<Expression> >::iterator i=call.arguments.begin(); i!=call.arguments.end(); ++i)
		(*i)->visit(*this);
}

void TraversingVisitor::visit(ExpressionStatement &expr)
{
	expr.expression->visit(*this);
}

void TraversingVisitor::visit(StructDeclaration &strct)
{
	strct.members.visit(*this);
}

void TraversingVisitor::visit(VariableDeclaration &var)
{
	if(var.init_expression)
		var.init_expression->visit(*this);
	if(var.array_size)
		var.array_size->visit(*this);
}

void TraversingVisitor::visit(InterfaceBlock &iface)
{
	iface.members.visit(*this);
}

void TraversingVisitor::visit(FunctionDeclaration &func)
{
	for(vector<NodePtr<VariableDeclaration> >::iterator i=func.parameters.begin(); i!=func.parameters.end(); ++i)
		(*i)->visit(*this);
	func.body.visit(*this);
}

void TraversingVisitor::visit(Conditional &cond)
{
	cond.condition->visit(*this);
	cond.body.visit(*this);
	cond.else_body.visit(*this);
}

void TraversingVisitor::visit(Iteration &iter)
{
	iter.init_statement->visit(*this);
	if(iter.condition)
		iter.condition->visit(*this);
	if(iter.loop_expression)
		iter.loop_expression->visit(*this);
	iter.body.visit(*this);
}

void TraversingVisitor::visit(Passthrough &pass)
{
	if(pass.subscript)
		pass.subscript->visit(*this);
}

void TraversingVisitor::visit(Return &ret)
{
	if(ret.expression)
		ret.expression->visit(*this);
}


Stage::Stage(StageType t):
	type(t),
	previous(0)
{ }


Module::Module():
	shared(SHARED)
{ }

} // namespace ProgramSyntax
} // namespace GL
} // namespace Msp
